/***** Includes *****/
/* Standard C Libraries */
#include <stdlib.h>

/* TI Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/sysbios/BIOS.h>
#include DeviceFamily_constructPath(driverlib/rf_prop_mailbox.h)

/* Application Header files */
#include "rf_layer.h"
#include "rf/RFQueue.h"
#include "smartrf_settings/smartrf_settings.h"
#include <ti/devices/cc13x0/driverlib/sys_ctrl.h>
#include "board/CC1310_LAUNCHXL.h"

/***** Prototypes *****/
static void callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e);

/***** Variable declarations *****/
static RF_Object rfObject;
static RF_Handle rfHandle;

/* Buffer which contains all Data Entries for receiving data.
 * Pragmas are needed to make sure this buffer is 4 byte aligned (requirement from the RF Core) */
#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN (rxDataEntryBuffer, 4);
static uint8_t
rxDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                  ENCODED_DATA_LEN,
                                                  NUM_APPENDED_BYTES)];
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment = 4
static uint8_t
rxDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                  ENCODED_DATA_LEN,
                                                  NUM_APPENDED_BYTES)];
#elif defined(__GNUC__)
static uint8_t
rxDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                  ENCODED_DATA_LEN,
                                                  NUM_APPENDED_BYTES)]
                                                  __attribute__((aligned(4)));
#else
#error This compiler is not supported.
#endif

/* Receive dataQueue for RF Core to fill in data */
static dataQueue_t dataQueue;
static rfc_dataEntryGeneral_t* currentDataEntry;

/* This buffer is not directly accessed by the application */
static RFMailboxMsgObj rf_mailboxBuffer[NUMMSGS];
Mailbox_Handle rf_mbxHandle;

Mailbox_Params mbxParams;
Mailbox_Struct mbxStruct;

#if AMP_ENABLED

PIN_Handle ampPinHandle;
PIN_State ampPinState;

#define Board_HGM             CC1310_LAUNCHXL_DIO28_ANALOG
#define Board_LNA_EN          CC1310_LAUNCHXL_DIO29_ANALOG
#define Board_PA_EN           CC1310_LAUNCHXL_DIO30_ANALOG

/*
Application button pin configuration table.
Buttons interrupts are configured to trigger on falling edge.
*/
PIN_Config ampPinTable[] =
{
    Board_HGM | PIN_GPIO_OUTPUT_EN | PIN_GPIO_HIGH | PIN_PUSHPULL | PIN_DRVSTR_MAX,
    Board_LNA_EN | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,
    Board_PA_EN | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX,

    PIN_TERMINATE,
};

#endif

void *packRxThread(void *arg0)
{

    /* Construct a Mailbox instance */
    Mailbox_Params_init(&mbxParams);
    mbxParams.buf = (Ptr)rf_mailboxBuffer;
    mbxParams.bufSize = sizeof(rf_mailboxBuffer);
    Mailbox_construct(&mbxStruct, sizeof(RFMsgObj), NUMMSGS, &mbxParams, NULL);
    rf_mbxHandle = Mailbox_handle(&mbxStruct);

    RF_Params rfParams;
    RF_Params_init(&rfParams);

    if( RFQueue_defineQueue(&dataQueue,
                            rxDataEntryBuffer,
                            sizeof(rxDataEntryBuffer),
                            NUM_DATA_ENTRIES,
                            ENCODED_DATA_LEN + NUM_APPENDED_BYTES))
    {
        /* Failed to allocate space for all data entries */
        while(1);
    }

#if AMP_ENABLED
    /* Open Button pins */
    ampPinHandle = PIN_open(&ampPinState, ampPinTable);
    //Assert_isTrue(buttonPinHandle != NULL, NULL);

    /* Configure the pins to control the range extender */
    IOCPortConfigureSet(Board_LNA_EN, IOC_PORT_RFC_GPO0, IOC_IOMODE_NORMAL);
    IOCPortConfigureSet(Board_PA_EN, IOC_PORT_RFC_GPO1, IOC_IOMODE_NORMAL);

    /* RX High Gain Mode */
    PIN_setOutputValue(ampPinHandle, Board_HGM, 1);
    PIN_setOutputValue(ampPinHandle, Board_LNA_EN, 1);
    PIN_setOutputValue(ampPinHandle, Board_PA_EN, 0);

    /* TX High Gain Mode */
    //PIN_setOutputValue(ampPinHandle, Board_HGM, 1);
    //PIN_setOutputValue(ampPinHandle, Board_LNA_EN, 0);
    //PIN_setOutputValue(ampPinHandle, Board_PA_EN, 1);
#endif

    /* Modify CMD_PROP_RX command for application needs */
    /* Set the Data Entity queue for received data */
    RF_cmdPropRx.pQueue = &dataQueue;
    /* Implement packet length filtering to avoid PROP_ERROR_RXBUF */
    RF_cmdPropRx.maxPktLen = ENCODED_DATA_LEN;
    RF_cmdPropRx.pktConf.bRepeatOk = 0x1;
    RF_cmdPropRx.pktConf.bRepeatNok = 0x1;
    RF_cmdPropRx.pktConf.bUseCrc = 0x0;
    RF_cmdPropRx.rxConf.bIncludeHdr = 0x0;
    RF_cmdPropRx.rxConf.bAppendRssi = 0x1;
    RF_cmdPropRx.rxConf.bAppendStatus = 0x0;

    /* Request access to the radio */
#if defined(DeviceFamily_CC26X0R2)
    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioSetup, &rfParams);
#else
    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);
#endif// DeviceFamily_CC26X0R2

    /* Set the frequency */
    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);

    /* Enter RX mode and stay forever in RX */
    RF_EventMask terminationReason = RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropRx,
                                               RF_PriorityHigh, &callback,
                                               RF_EventRxEntryDone);

    switch(terminationReason)
    {
        case RF_EventLastCmdDone:
            // A stand-alone radio operation command or the last radio
            // operation command in a chain finished.
            //printf("RF_EventLastCmdDone\r\n");
            break;
        case RF_EventCmdCancelled:
            // Command cancelled before it was started; it can be caused
            // by RF_cancelCmd() or RF_flushCmd().
            //printf("RF_EventCmdCancelled\r\n");
            break;
        case RF_EventCmdAborted:
            // Abrupt command termination caused by RF_cancelCmd() or
            // RF_flushCmd().
            //printf("RF_EventCmdAborted\r\n");
            break;
        case RF_EventCmdStopped:
            // Graceful command termination caused by RF_cancelCmd() or
            // RF_flushCmd().
            //printf("RF_EventCmdStopped\r\n");
            break;
        default:
            // Uncaught error event
            //printf("default1\r\n");
            SysCtrlSystemReset();
            while(1);
    }

    uint32_t cmdStatus = ((volatile RF_Op*)&RF_cmdPropRx)->status;
    switch(cmdStatus)
    {
        case PROP_DONE_OK:
            // Packet received with CRC OK
            //printf("PROP_DONE_OK\r\n");
            break;
        case PROP_DONE_RXERR:
            // Packet received with CRC error
            //printf("PROP_DONE_RXERR\r\n");
            break;
        case PROP_DONE_RXTIMEOUT:
            // Observed end trigger while in sync search
            //printf("PROP_DONE_RXTIMEOUT\r\n");
            break;
        case PROP_DONE_BREAK:
            // Observed end trigger while receiving packet when the command is
            // configured with endType set to 1
            //printf("PROP_DONE_BREAK\r\n");
            break;
        case PROP_DONE_ENDED:
            // Received packet after having observed the end trigger; if the
            // command is configured with endType set to 0, the end trigger
            // will not terminate an ongoing reception
            //printf("PROP_DONE_ENDED\r\n");
            break;
        case PROP_DONE_STOPPED:
            // received CMD_STOP after command started and, if sync found,
            // packet is received
            //printf("PROP_DONE_STOPPED\r\n");
            break;
        case PROP_DONE_ABORT:
            // Received CMD_ABORT after command started
            //printf("PROP_DONE_ABORT\r\n");
            break;
        case PROP_ERROR_RXBUF:
            // No RX buffer large enough for the received data available at
            // the start of a packet
            //printf("PROP_ERROR_RXBUF\r\n");
            break;
        case PROP_ERROR_RXFULL:
            // Out of RX buffer space during reception in a partial read
            //printf("PROP_ERROR_RXFULL\r\n");
            break;
        case PROP_ERROR_PAR:
            // Observed illegal parameter
            //printf("PROP_ERROR_PAR\r\n");
            break;
        case PROP_ERROR_NO_SETUP:
            // Command sent without setting up the radio in a supported
            // mode using CMD_PROP_RADIO_SETUP or CMD_RADIO_SETUP
            //printf("PROP_ERROR_NO_SETUP\r\n");
            break;
        case PROP_ERROR_NO_FS:
            // Command sent without the synthesizer being programmed
            //printf("PROP_ERROR_NO_FS\r\n");
            break;
        case PROP_ERROR_RXOVF:
            // RX overflow observed during operation
            //printf("PROP_ERROR_RXOVF\r\n");
            break;
        default:
            // Uncaught error event - these could come from the
            // pool of states defined in rf_mailbox.h
            //printf("default2\r\n");
            SysCtrlSystemReset();
            while(1);
    }

    SysCtrlSystemReset();
    while(1);
}


RFMsgObj RFmsg;
void callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e)
{
    if (e & RF_EventRxEntryDone)
    {
        /* Get current unhandled data entry */
        currentDataEntry = RFQueue_getDataEntry();

        memcpy(RFmsg.buf, (uint8_t*)(&currentDataEntry->data), currentDataEntry->length);

        if (Mailbox_post(rf_mbxHandle, &RFmsg, BIOS_NO_WAIT)) {

            ////printf("1Mailbox Write \r\n");
        }
        else {

            ////printf("1Mailbox Write Failed\r\n");
        }

        RFQueue_nextEntry();
    }
}
